#pragma once

#include "mapsearchnode.h"

class dtNavMesh;
class dtNavMeshQuery;
class Room;
class Human;
class ColliderComponent;

enum TILE_STATE
{
   TILE_STATE_OPENED_COST0 = 0,
   TILE_STATE_OPENED_COST1 = 1,
   TILE_STATE_OPENED_COST2 = 2,
   TILE_STATE_OPENED_COST3 = 3,
   TILE_STATE_OPENED_COST4 = 4,
   TILE_STATE_OPENED_COST5 = 5,
   TILE_STATE_CLOSED = 9
};

enum NavMeshSearchState_e
{
   kNavMesh_Searching = 0,
   kNavMesh_SearchOk = 1,
   kNavMesh_SearchFailed = 2,
};

struct CellNode
{
    ColliderComponent* collider;
    list_head entry;
    CellNode* next = nullptr;
    #ifdef DEBUG
    int x = 0;
    int y = 0;
    #endif
};

struct MovePathPoint
{
    a8::Vec2 pos;
    float distance = 0;
};

struct FindPathStatus
{
    long long frameno = 0;
    Human* hum = nullptr;
    a8::Vec2 start_pos;
    a8::Vec2 end_pos;

    int search_step = 0;
    int max_step_num = 0;
    int path_index = 0;
    std::vector<MovePathPoint> out_points;
    list_head entry;
    int search_state = AStarSearch<MapSearchNode>::SEARCH_STATE_SEARCHING;
    AStarSearch<MapSearchNode> astar_search;

    int navmesh_search_state = kNavMesh_Searching;
    dtNavMeshQuery* navmesh_query = nullptr;

    FindPathStatus()
    {
        INIT_LIST_HEAD(&entry);
    }
};

class MapService
{
 public:
    MapService();
    ~MapService();

    void Init(int width, int height, int cell_width);
    void UnInit();

    void AddCollider(ColliderComponent* collider);
    void RemoveCollider(ColliderComponent* collider);
    void GetColliders(Room* room,
                      float world_x,
                      float world_y,
                      std::set<ColliderComponent*>& colliders);
    int GetMap(int x, int y);

    FindPathStatus* AddFindPathRequest(Human* hum,
                                       const a8::Vec2& start_pos,
                                       const a8::Vec2& end_pos,
                                       int max_step_num);
    void DelFindPathRequest(Human* hum, FindPathStatus* status);
    void FindPathUpdate(FindPathStatus* find_status);
    bool FindOk(FindPathStatus* find_status);
    bool FindFailed(FindPathStatus* find_status);
    bool MoveDone(FindPathStatus* find_status);
    void DoMove(FindPathStatus* find_status);
    void SetNavMesh(dtNavMesh* navmesh) { navmesh_ = navmesh; };

 private:
    int GetGridId(float world_x, float world_y);
    bool IsValidPos(int x, int y);
    void UpdateAStar(FindPathStatus* find_status);
    void UpdateNavmesh(FindPathStatus* find_status);

 private:
    bool use_navmesh_ = false;
    dtNavMesh* navmesh_ = nullptr;
    list_head* map_cells_ = nullptr;
    int map_width_ = 0;
    int map_height_ = 0;
    int cell_width_ = 0;
    int max_grid_id_ = 0;
    int grid_offset_arr_[9] = {0};
    list_head findpath_list_;
    std::map<Human*, FindPathStatus*> findpath_requests_hash_;
    std::map<ColliderComponent*, CellNode*> node_hash_;
};
