#include "precompile.h"

#include <a8/udplog.h>

#include "mapinstance.h"
#include "metamgr.h"
#include "navmeshhelper.h"
#include "collider.h"
#include "entity.h"

void BuildContext::doResetLog()
{
    a8::UdpLog::Instance()->Debug("doResetLog", {});
}

void BuildContext::doLog(const rcLogCategory category, const char* msg, const int len)
{
    a8::UdpLog::Instance()->Debug("doLog category:%d msg:%s",
                                  {
                                   category,
                                   msg
                                  });
}

void BuildContext::doResetTimers()
{
    a8::UdpLog::Instance()->Debug("doResetTimers", {});
}

void BuildContext::doStartTimer(const rcTimerLabel label)
{
    a8::UdpLog::Instance()->Debug("doStartTimer lable:%d", {label});
}

void BuildContext::doStopTimer(const rcTimerLabel label)
{
    a8::UdpLog::Instance()->Debug("doStopTimer lable:%d", {label});
}

int BuildContext::doGetAccumulatedTime(const rcTimerLabel label) const
{
    return 0;
}

void NavMeshHelper::OutputObjFile(MapInstance* map_instance)
{
    std::vector<a8::Vec2> vertexs;
    std::vector<std::tuple<int, int, int>> faces;
    vertexs.reserve(10000);
    for (auto& pair : map_instance->uniid_hash_) {
        for (ColliderComponent* collider : *pair.second->GetColliders()) {
            if (collider->type == CT_Aabb) {
                AabbCollider* aabb_box = (AabbCollider*)collider;
                {
                    a8::Vec2 vert = collider->owner->GetPos() + aabb_box->_min;
                    vert.x = vert.x - map_instance->map_meta_->i->map_width() / 2.0f;
                    vert.y = vert.y - map_instance->map_meta_->i->map_height() / 2.0f;

                    vertexs.push_back(vert);
                    vert.y += aabb_box->_max.y - aabb_box->_min.y;
                    vertexs.push_back(vert);
                    vert.x += aabb_box->_max.x - aabb_box->_min.x;
                    vertexs.push_back(vert);
                    vert.y -= aabb_box->_max.y - aabb_box->_min.y;
                    vertexs.push_back(vert);
                }
                //0 1 2
                faces.push_back
                    (std::make_tuple
                     (
                      vertexs.size() - 4 + 1,
                      vertexs.size() - 3 + 1,
                      vertexs.size() - 2 + 1
                      ));
                //0 2 3
                faces.push_back
                    (std::make_tuple
                     (
                      vertexs.size() - 4 + 1,
                      vertexs.size() - 2 + 1,
                      vertexs.size() - 1 + 1
                      ));
            }
        }
    }
    {
        std::string filename = a8::Format("%s.obj", {map_instance->map_tpl_name_});
        FILE* fp = fopen(filename.c_str(), "wb");
        #if 1
        {
            vertexs.clear();
            faces.clear();
            {
                a8::Vec2 vert;
                //vert.x = 0 - map_instance->map_meta_->i->map_width() / 2.0f;
                //vert.y = 0 - map_instance->map_meta_->i->map_height() / 2.0f;

                vertexs.push_back(vert);
                vert.y += map_instance->map_meta_->i->map_height();
                vertexs.push_back(vert);
                vert.x += map_instance->map_meta_->i->map_width();
                vertexs.push_back(vert);
                vert.y -= map_instance->map_meta_->i->map_height();
                vertexs.push_back(vert);

                //0 1 2
                faces.push_back
                    (std::make_tuple
                     (
                      vertexs.size() - 4 + 1,
                      vertexs.size() - 3 + 1,
                      vertexs.size() - 2 + 1
                      ));
                //0 2 3
                faces.push_back
                    (std::make_tuple
                     (
                      vertexs.size() - 4 + 1,
                      vertexs.size() - 2 + 1,
                      vertexs.size() - 1 + 1
                      ));
            }
        }
        #endif
        for (auto& vert : vertexs) {
            std::string data = a8::Format("v %f %f %f\r\n",
                                          {
                                           vert.x,
                                           0,
                                           vert.y,
                                          });
            fwrite(data.data(), 1, data.size(), fp);
        }
        for (auto& tuple : faces) {
            std::string data = a8::Format("f %d %d %d\r\n",
                                          {
                                           std::get<0>(tuple),
                                           std::get<1>(tuple),
                                           std::get<2>(tuple)
                                          });
            fwrite(data.data(), 1, data.size(), fp);
        }
        fclose(fp);
    }
}
